#include "EventHandler.h"

#include <QtGui>
#include <math.h>

EventHandler::EventHandler()
    : m_hairDirection(M_PI/2.0),
    m_maxBrushLength(60.0),
    m_defaultBrushWidth(10.0)
{
}

void EventHandler::handleTabletEvent( QTabletEvent *event ) {
    m_lastPositions.append(event->hiResGlobalPos());
    QPointF diff = m_lastPositions.last() - m_lastPositions[0];
    if(sqrt( pow(diff.x(), 2.0) + pow(diff.y(), 2.0) ) > 4)
        m_lastPositions.removeAt(0);

    if(m_brushPosition == QPointF(0,0)) {
        m_brushPosition = event->hiResGlobalPos();
        initBrush(event->pressure());
        return;
    }
    m_brushPosition = event->hiResGlobalPos();
    diff = m_lastPositions.last() - m_lastPositions[0];

    double brushWidth;
    if(event->pressure() < 0.01) { // I'm only positioning the brush, not touching the medium yet
        if(diff.y() == 0)
            m_hairDirection = M_PI / 2.0;
        m_hairDirection = angleFromDistance(diff);
        brushWidth = m_defaultBrushWidth;
        m_contactAreaRadius = m_defaultBrushWidth;
    }
    else {
        brushWidth = m_defaultBrushWidth * (0.5 + event->pressure() * 1.3);
        if(m_contactAreaRadius < 0.1) {
            m_hairDirection = m_hairDirection + M_PI;
            while(m_hairDirection > 2 * M_PI)
                m_hairDirection -= 2 * M_PI;
        }
        const double currentAngle = angleFromDistance(diff);
        double delta = m_hairDirection - currentAngle;
        // if running in opposite direction, don't rotate but swap.
        if(qAbs(delta) > M_PI * (170 / 180.)) {
            m_contactAreaRadius *= 0.85;
        }
        else {
            // dampen change
            if(delta > 0.08) delta = 0.08;
            if(delta < -0.08) delta = -0.08;
            m_hairDirection = m_hairDirection - delta;

            m_contactAreaRadius = m_defaultBrushWidth * (0.5 + event->pressure() * 1.3);
        }
    }

    // first calculate top of brush using angle and length
    // then calculate the 2 points based on the width of the brush
    const double tmpCos = cos(m_hairDirection);
    const double tmpSin = sin(m_hairDirection);
    QLineF brush(m_brushPosition, m_brushPosition +
            QPointF( tmpCos * m_contactAreaRadius, tmpSin * m_contactAreaRadius));
    QLineF tip1(brush.p2(), brush.p2() + QPointF(brush.dy(), -brush.dx()));
    tip1.setLength(brushWidth / 2.0);
    QLineF tip2(brush.p2(), brush.p2() + QPointF(-brush.dy(), brush.dx()));
    tip2.setLength(brushWidth / 2.0);
    m_brushEndPoints.clear();
    m_brushEndPoints.append(tip1.p2());
    m_brushEndPoints.append(tip2.p2());
}

void EventHandler::initBrush(qreal pressure) {
    m_hairDirection = M_PI / 2.0;
    m_contactAreaRadius = m_maxBrushLength * pressure;

    double width = m_defaultBrushWidth * (0.5 + pressure * 1.3);

    // first calculate top of brush using angle and length
    // then calculate the 2 points based on the width of the brush
    QPointF topOfBrush = m_brushPosition + QPointF(0, m_contactAreaRadius); // easy, its 90 degrees ;)
    QPointF left = topOfBrush - QPointF(width / 2.0, 0);
    QPointF right = topOfBrush + QPointF(width / 2.0, 0);
    m_brushEndPoints.clear();
    m_brushEndPoints.append(left);
    m_brushEndPoints.append(right);
}

double EventHandler::angleFromDistance(const QPointF &point) {
    double a = atan2(point.y(), point.x());
    a += M_PI;
    while(a > 2 * M_PI)
        a -= 2 * M_PI;
    return a;
}
